"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CodeGenerator = void 0;
const os_1 = require("os");
const Api = __importStar(require("./api"));
class CodeGenerator {
    constructor(entryPoint, option) {
        this.entryPoint = entryPoint;
        this.option = option;
        this.rootSchema = Api.FileSystem.loadJsonOrYaml(entryPoint);
        this.resolvedReferenceDocument = Api.ResolveReference.resolve(entryPoint, entryPoint, JSON.parse(JSON.stringify(this.rootSchema)));
        this.parser = this.createParser();
    }
    createParser() {
        var _a;
        return new Api.OpenApiTools.Parser(this.entryPoint, this.rootSchema, this.resolvedReferenceDocument, (_a = this.option) === null || _a === void 0 ? void 0 : _a.convertOption);
    }
    /**
     * Validate the OpenAPI Schema
     */
    validateOpenApiSchema(option) {
        if (!option) {
            Api.Validator.validate(this.resolvedReferenceDocument);
        }
        else {
            Api.Validator.validate(this.resolvedReferenceDocument, option.logger);
        }
    }
    /**
     * Provides TypeScript typedefs generated from OpenAPI Schema.
     *
     * @param generatorTemplate Template for when you want to change the code following a type definition
     * @returns String of generated code
     */
    generateTypeDefinition(generatorTemplates, allowOperationIds) {
        const create = () => {
            const statements = this.parser.getOpenApiTypeDefinitionStatements();
            generatorTemplates === null || generatorTemplates === void 0 ? void 0 : generatorTemplates.forEach(generatorTemplate => {
                const payload = this.parser.getCodeGeneratorParamsArray(allowOperationIds);
                const extraStatements = Api.TsGenerator.Utils.convertIntermediateCodes(generatorTemplate.generator(payload, generatorTemplate.option));
                statements.push(...extraStatements);
            });
            return statements;
        };
        return [Api.OpenApiTools.Comment.generateLeading(this.resolvedReferenceDocument), Api.TsGenerator.generate(create)].join(os_1.EOL + os_1.EOL + os_1.EOL);
    }
    /**
     * Generate code using a template
     *
     * @param generatorTemplate
     * @returns String of generated code
     */
    generateCode(generatorTemplates, allowOperationIds) {
        const payload = this.parser.getCodeGeneratorParamsArray(allowOperationIds);
        const create = () => {
            return generatorTemplates
                .map(generatorTemplate => {
                return Api.TsGenerator.Utils.convertIntermediateCodes(generatorTemplate === null || generatorTemplate === void 0 ? void 0 : generatorTemplate.generator(payload, generatorTemplate.option));
            })
                .flat();
        };
        return [Api.OpenApiTools.Comment.generateLeading(this.resolvedReferenceDocument), Api.TsGenerator.generate(create)].join(os_1.EOL + os_1.EOL + os_1.EOL);
    }
    /**
     * Provides parameters extracted from OpenApi Schema
     */
    getCodeGeneratorParamsArray(allowOperationIds) {
        return this.parser.getCodeGeneratorParamsArray(allowOperationIds);
    }
    /**
     * Provides types for parameters for Templates.ApiClient.
     *
     * This API will be moved to Templates in the future.
     */
    getAdditionalTypeDefinitionCustomCodeGenerator() {
        return {
            generator: () => this.parser.getAdditionalTypeStatements(),
        };
    }
}
exports.CodeGenerator = CodeGenerator;
//# sourceMappingURL=index.js.map