import { EOL } from "os";
import * as Api from "./api";
export class CodeGenerator {
    constructor(entryPoint, option) {
        this.entryPoint = entryPoint;
        this.option = option;
        this.rootSchema = Api.FileSystem.loadJsonOrYaml(entryPoint);
        this.resolvedReferenceDocument = Api.ResolveReference.resolve(entryPoint, entryPoint, JSON.parse(JSON.stringify(this.rootSchema)));
        this.parser = this.createParser();
    }
    createParser() {
        var _a;
        return new Api.OpenApiTools.Parser(this.entryPoint, this.rootSchema, this.resolvedReferenceDocument, (_a = this.option) === null || _a === void 0 ? void 0 : _a.convertOption);
    }
    /**
     * Validate the OpenAPI Schema
     */
    validateOpenApiSchema(option) {
        if (!option) {
            Api.Validator.validate(this.resolvedReferenceDocument);
        }
        else {
            Api.Validator.validate(this.resolvedReferenceDocument, option.logger);
        }
    }
    /**
     * Provides TypeScript typedefs generated from OpenAPI Schema.
     *
     * @param generatorTemplate Template for when you want to change the code following a type definition
     * @returns String of generated code
     */
    generateTypeDefinition(generatorTemplates, allowOperationIds) {
        const create = () => {
            const statements = this.parser.getOpenApiTypeDefinitionStatements();
            generatorTemplates === null || generatorTemplates === void 0 ? void 0 : generatorTemplates.forEach(generatorTemplate => {
                const payload = this.parser.getCodeGeneratorParamsArray(allowOperationIds);
                const extraStatements = Api.TsGenerator.Utils.convertIntermediateCodes(generatorTemplate.generator(payload, generatorTemplate.option));
                statements.push(...extraStatements);
            });
            return statements;
        };
        return [Api.OpenApiTools.Comment.generateLeading(this.resolvedReferenceDocument), Api.TsGenerator.generate(create)].join(EOL + EOL + EOL);
    }
    /**
     * Generate code using a template
     *
     * @param generatorTemplate
     * @returns String of generated code
     */
    generateCode(generatorTemplates, allowOperationIds) {
        const payload = this.parser.getCodeGeneratorParamsArray(allowOperationIds);
        const create = () => {
            return generatorTemplates
                .map(generatorTemplate => {
                return Api.TsGenerator.Utils.convertIntermediateCodes(generatorTemplate === null || generatorTemplate === void 0 ? void 0 : generatorTemplate.generator(payload, generatorTemplate.option));
            })
                .flat();
        };
        return [Api.OpenApiTools.Comment.generateLeading(this.resolvedReferenceDocument), Api.TsGenerator.generate(create)].join(EOL + EOL + EOL);
    }
    /**
     * Provides parameters extracted from OpenApi Schema
     */
    getCodeGeneratorParamsArray(allowOperationIds) {
        return this.parser.getCodeGeneratorParamsArray(allowOperationIds);
    }
    /**
     * Provides types for parameters for Templates.ApiClient.
     *
     * This API will be moved to Templates in the future.
     */
    getAdditionalTypeDefinitionCustomCodeGenerator() {
        return {
            generator: () => this.parser.getAdditionalTypeStatements(),
        };
    }
}
//# sourceMappingURL=index.js.map